demos = list()

demo1 = '''开始日期 = '2024-04-10'
结束日期 = '2024-04-10'
正回购资产_明细 = 资产(['债券'], ['久期', '持仓市值']) in 产品(['净值类'], ['总资产', '净资产', '杠杆率'])
正回购资产合计 = agg(正回购资产_明细, ['产品代码', '产品简称'], {'持仓市值': 'sum', '净资产': 'first', '杠杆率': 'first'})
持仓占比 = alias(正回购资产合计.持仓市值 / 正回购资产合计.净资产, '正回购占比')
监控规则 = {'触发超限': '正回购占比 > 0.44', '触发预警': '(正回购占比 <= 0.44) & (正回购占比 > 0.41)', '正常': '正回购占比 <= 0.41'}
'''

demo2 = '''开始日期 = '2024-04-10'
结束日期 = '2024-04-10'
资产指标库 = 指标('持仓明细', ['持仓市值'], {'PORT_TAGS': ['公募开放式', '封闭式'], 'SEC_TAGS': ['债券', '债券回购']})
'''

demo3 = '''开始日期 = '2024-04-10'
结束日期 = '2024-04-20'
资产指标库 = 指标('持仓明细', ['持仓市值'], {'PORT_TAGS': ['公募开放式', '封闭式'], 'SEC_TAGS': ['债券', '债券回购']})
最大持仓市值 = max(资产指标库, '持仓市值')
最小持仓市值 = min(资产指标库, '持仓市值')
平均持仓市值 = mean(资产指标库, '持仓市值')
持仓个数 = count(资产指标库, '持仓市值')
持仓中位数 = median(资产指标库, '持仓市值')
排序后资产指标库 = sort(资产指标库, ['持仓市值'], '降序')
'''

demo4 = '''开始日期 = '2024-04-10'
结束日期 = '2024-04-20'
资产指标库 = 指标('持仓明细', ['持仓市值'], {'PORT_TAGS': ['公募开放式', '封闭式'], 'SEC_TAGS': ['债券', '债券回购']})
债券库 = filter(资产指标库, '债券 == "是"')
回购库 = filter(资产指标库, '债券回购 == "是"')
全部库 = concat(债券库, 回购库)
'''

demo5 = '''开始日期 = '2024-04-10'
结束日期 = '2024-04-20'
资产指标库 = 指标('持仓明细', ['持仓市值'], {'PORT_TAGS': ['公募开放式', '封闭式'], 'SEC_TAGS': ['债券', '债券回购']})
排序后指标库 = sort(资产指标库, ['持仓市值'], '降序')
前十大持仓资产 = limit(排序后指标库, 10)
'''

demo6 = '''开始日期 = '2024-04-10'
结束日期 = '2024-04-20'
PORT_TAGS = ['公募开放式', '封闭式']
SEC_TAGS = ['债券', '债券回购']
资产指标库 = 指标('持仓明细', ['持仓市值'], {'PORT_TAGS': PORT_TAGS, 'SEC_TAGS': SEC_TAGS})
排序后指标库 = sort(资产指标库, ['持仓市值'], '降序')
前十大持仓资产 = limit(排序后指标库, 10)
'''

demo7 = '''净资产情况 = 指标('产品估值', ['产品代码', '单位净值', '净资产', '上月末净资产'], {'PORT_TAGS': "固收部门", 'START_DATE': '2024-04-01', 'END_DATE': '2024-04-10'})
产品情况 = 指标('产品要素', ['产品代码', '产品简称', '开放类型', '基金经理'], {'PORT_TAGS': "固收部门", 'START_DATE': '2024-04-01', 'END_DATE': '2024-04-10'})
结果 = merge(净资产情况, 产品情况, ['产品代码'], 'left')
'''

demo8 = '''大类资产占比 = 指标('产品大类资产占比', [], {'PORT_IDS': "P01,P02", 'START_DATE': '2024-04-10', 'END_DATE': '2024-04-10'})
'''

demo9 = '''七日年化收益情况 = 指标('产品估值', ['产品代码', '七日年化收益率'], {'PORT_TAGS': "现金管理类", 'START_DATE': '2024-04-01', 'END_DATE': '2024-04-10'})
'''

demo10 = '''产品收益情况 = 指标('产品收益', [], {'START_DATE': '2024-04-10', 'END_DATE': '2024-04-10'})
'''

demo11 = '''产品风险情况 = 指标('产品风险', [], {'PORT_TAGS': "净值类", 'START_DATE': '2024-04-10', 'END_DATE': '2024-04-10'})
'''

demo12 = '''七日年化收益情况 = 指标('产品估值', ['产品代码', '七日年化收益率'], {'PORT_IDS': "P12", 'START_DATE': '2024-04-01', 'END_DATE': '2024-04-10'})
'''

demo18 = '''产品风险情况 = 指标('产品风险', ['产品代码', 'VaR95', 'VaR99', '近一月最大回撤', '近一月波动率'], {'PORT_IDS': 'P01,P02,P03', 'START_DATE': '2024-04-10', 'END_DATE': '2024-04-10'})
'''

demos.append(demo1)
demos.append(demo2)
demos.append(demo3)
demos.append(demo4)
demos.append(demo5)
demos.append(demo6)
demos.append(demo7)
demos.append(demo8)
demos.append(demo9)
demos.append(demo10)
demos.append(demo11)
demos.append(demo12)
demos.append(demo18)
